@@ -65,6 +65,10 @@ SHORT_NAMES = {'quit': 'Quit',
                'delegate': 'Delegate task',
                'refresh-screen': 'Refresh screen',
                'reload-tasks': 'Reload todo files',
+               'comp-next': 'Next completion option',
+               'comp-prev': 'Previous completion option',
+               'comp-use': 'Use selected completion option',
+               'comp-close': 'Close the completion list',
                }
 
 
@@ -252,7 +256,7 @@ class RemappedScrollPanel(ScrollPanel):
         self.SCROLL_MARGIN = self.app.scroll_margin
 
 
-class TaskList(ScrollPanel):
+class TaskList(RemappedScrollPanel):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.tasks = []
@@ -632,10 +636,21 @@ class RemappedInputLine(InputLine):
 
 
 class ContextCompletion(Completion):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.KEYS_NEXT_ALTERNATIVE = [key for key, value in self.app.completion_key_mapping.items()
+                                      if value == 'comp-next']
+        self.KEYS_PREVIOUS_ALTERNATIVE = [key for key, value in self.app.completion_key_mapping.items()
+                                          if value == 'comp-prev']
+        self.KEYS_SELECT_ALTERNATIVE = [key for key, value in self.app.completion_key_mapping.items()
+                                        if value == 'comp-use']
+        self.KEYS_CANCEL_SUGGESTIONS = [key for key, value in self.app.completion_key_mapping.items()
+                                        if value == 'comp-close']
+
     def close(self):
         retval = super().close()
         if retval:
-            self.app.paint()
+            self.app.paint(True)
         return retval
 
     def update(self, y, x):
@@ -671,7 +686,8 @@ class SearchBar(RemappedInputLine):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.previous_text = self.text
-        self.completion = ContextCompletion(self)
+        if self.app.use_completion:
+            self.completion = ContextCompletion(self)
 
     def paint(self, clear=False):
         self.border = Panel.BORDER_NONE
@@ -791,7 +807,8 @@ class TaskEditor(UserInput):
         self.task = task
         self.on_accept = self.save_changes
         self.on_cancel = lambda: self.app.show_cursor(False)
-        self.editor.completion = ContextCompletion(self.editor)
+        if self.app.use_completion:
+            self.editor.completion = ContextCompletion(self.editor)
 
     def save_changes(self, text):
         self.app.show_cursor(False)
@@ -915,9 +932,9 @@ class Selector(RemappedScrollPanel):
         maxheight, maxwidth = self.app.size()
         labelwidth = max([len(self.make_label(item)) for item in self.items]) \
                    + len(self.indicator(True)) \
-                   + len(str(len(self.items)))
-        w = min(2*maxwidth//3 if maxwidth > 11 else maxwidth, labelwidth)
-        h = min(4*maxheight//5 if maxheight > 9 else maxheight, len(self.items)+2)
+                   + len(str(len(self.items))) + 3
+        w = min(maxwidth, labelwidth)
+        h = min(maxheight-2, len(self.items)+2)
         self.resize(h, w)
         self.move((maxheight-h)//2, (maxwidth-w)//2)
 
@@ -1017,17 +1034,29 @@ class HelpScreen(RemappedScrollPanel):
         edt_nav_fncs = ['go-left', 'go-right', 'go-bol', 'go-eol']
         edt_edt_fncs = ['del-left', 'del-right', 'del-to-bol']
         edt_meta_fncs = ['cancel', 'submit-input']
+        edt_comp_fncs = ['comp-next', 'comp-prev', 'comp-use', 'comp-close']
         lines += [('', ''), ('', ''), (tr('TASK EDITING'), ''), ('', ''), (tr('General'), '')]
         lines += [(name, key) for name, key in self.collect_name_fnc(edt_meta_fncs, self.app.editor_key_mapping)]
         lines += [('', ''), (tr('Navigation'), '')]
         lines += [(name, key) for name, key in self.collect_name_fnc(edt_nav_fncs, self.app.editor_key_mapping)]
         lines += [('', ''), (tr('Deletion'), '')]
         lines += [(name, key) for name, key in self.collect_name_fnc(edt_edt_fncs, self.app.editor_key_mapping)]
+        lines += [('', ''), (tr('Completion'), '')]
+        lines += [(name, key) for name, key in self.collect_name_fnc(edt_comp_fncs, self.app.completion_key_mapping)]
 
         self.maxnamelen = max([len(n) for n, _ in lines])
 
         self.items = lines
         self.border = Panel.BORDER_ALL
+        self.SCROLL_MARGIN = 1000
+
+    def resize(self, *args):
+        super().resize(*args)
+        self.scroll()
+
+    def scroll(self):
+        self.cursor = max((self.list_height-1)//2, self.cursor)
+        super().scroll()
 
     def do_paint_item(self, y, x, maxwidth, is_selected, item):
         attrs = common.SETTING_COL_NORMAL
@@ -1042,7 +1071,9 @@ class HelpScreen(RemappedScrollPanel):
                 yield name, key
 
     def handle_key(self, key):
-        if str(key) in self.app.key_mapping and self.app.key_mapping[str(key)] in ['quit', 'cancel']:
+        fnc = self.app.key_mapping.get(str(key), None)
+
+        if fnc in ['quit', 'cancel']:
             self.destroy()
             self.app.focus.pop(-1)
             self.app.paint(True)
@@ -1158,6 +1189,8 @@ class CursesApplication(Application):
                                          common.SETTING_SCROLL_MARGIN)
         self.safe_save = conf.bool(common.SETTING_GROUP_GENERAL,
                                    common.SETTING_SAFE_SAVE, 'y')
+        self.use_completion = conf.bool(common.SETTING_GROUP_GENERAL,
+                                        common.SETTING_USE_COMPLETION, 'y')
 
         # external editor
         self.external_editor = self.conf.get(common.SETTING_GROUP_GENERAL,
@@ -1217,6 +1250,16 @@ class CursesApplication(Application):
             '<f6>': 'select-file',
             '<return>': 'submit-input',
             }
+        self.completion_key_mapping = {
+                '^C': 'comp-close',
+                Key.ESCAPE: 'comp-close',
+                Key.DOWN: 'comp-next',
+                Key.TAB: 'comp-next',
+                '^N': 'comp-next',
+                Key.UP: 'comp-prev',
+                '^P': 'comp-prev',
+                Key.RETURN: 'comp-use',
+            }
         self.functions = {
             'quit': self.do_quit,
             'nop': lambda: True,
@@ -1544,22 +1587,30 @@ class CursesApplication(Application):
 
     def load_key_configuration(self):
         logging.debug("Loading key configuration")
-        existing_functions = set(self.functions.keys()) | set(self.key_mapping.values())
         for item in self.conf[common.SETTING_GROUP_KEYS]:
+            target = None
             fnc = self.conf.get(common.SETTING_GROUP_KEYS, item, None)
-            if fnc is None or fnc not in existing_functions:
-                continue
+            for mapping in [self.key_mapping, self.editor_key_mapping, self.completion_key_mapping]:
+                if fnc in mapping.values():
+                    target = mapping
+            if target is None:
+                if fnc in self.functions:
+                    target = self.key_mapping
+                else:
+                    logging.warning(f"Cannot bind {item} to {fnc}: no such function")
+                    continue
 
             logging.debug(f"Trying to map {item} to {fnc}")
 
             if len(item) == 1:
-                self.key_mapping[item] = fnc
+                target[item] = fnc
             elif len(item) == 2 and item[0] == '^':
-                self.key_mapping[item.upper()] = fnc
+                target[item.upper()] = fnc
             elif item in Key.SPECIAL:
-                self.key_mapping[item] = fnc
+                target[item] = fnc
             else:
                 logging.error(f"Invalid key name '{item}' in configuration")
+
         to_exit = [k for k, fnc in self.key_mapping.items() if fnc == 'quit']
         if len(to_exit) == 0:
             logging.fatal(f"No key defined to exit pter.")
